--[[

This script used to encode 4 CIF resolution streams from the NTSC output
of the TW2835.  The TW2835 scales and composes 4 CIF images into a single
D1 image, which in then is divided into 4 streams which are each encoded
and streamed.  

Splitting of the single stream into four separate video inputs is done
using four NVPP objects each of which is connected to a single LVPP
object handling video port 0.  The display consists of a 2x2 grid of
either live or reconstructed preview that is similar to that of the 
TW2835 but can have an OSD overlay.

Streaming is done either with raw UDP packets or with RTP.

--]]

----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

BITRATE 	= BITRATE or {};

-- Number of live encoded channels
ENCCH		= ENCCH or 4;

-- Host IP and base port for udp live stream (port, port+1, port+2...)
-- Streaming protocol 0 for nothing, 1 for UDP, 2 for RTP
STREAMING 	= STREAMING or 2;
STORAGE     = STORAGE or 1;
SERVER      = SERVER or "10.10.10.207";
HOST 		= HOST or "192.168.40.38";
PORT 		= PORT or 554;
MAXPACKETSIZE = MAXPACKETSIZE or 1472;

-- show or hide bitrate 0 for off, 1 for on
SHOWRATE 	= SHOWRATE or 0;

-- video input mode svideo or composite
VIN 		= VIN or "svideo";
VOUT		= VOUT or "svideo";
FORMAT		= FORMAT or "480i"
-- Width and height of each live encoded stream
ENCW 		= ENCW or 352;
ENCH 		= ENCH or 240;

-- Geometry of input grid (typically 2x2 for TW2835)
ENCGRIDW 	= ENCGRIDW or 2;
ENCGRIDH 	= ENCGRIDH or 2;

-- Bitrate of each live encoded stream
BITRATE[0] 	= BITRATE[0] or 500000;
BITRATE[1] 	= BITRATE[1] or 500000;
BITRATE[2] 	= BITRATE[2] or 500000;
BITRATE[3] 	= BITRATE[3] or 500000;

-- Output filenames
OutputFileName = {};
OutputFileName[0] = "/media/sda1/vid3/dvr_test0";
OutputFileName[1] = "/media/sda1/vid3/dvr_test1";
OutputFileName[2] = "/media/sda1/vid3/dvr_test2";
OutputFileName[3] = "/media/sda1/vid3/dvr_test3";

-- Number of rendered images and their scaled size
OUTCH 		= OUTCH or 4;
OUTW 		= OUTW or 352;
OUTH 		= OUTH or 240;

-- Geometry of output grid (typically 2x2 for TW2835)
OUTGRIDW 	= OUTGRIDW or 2;
OUTGRIDH 	= OUTGRIDH or 2;

-- Preview mode (1 for live, 2 for reconstructed, 0 for off)
PREVIEW 	= PREVIEW or 1;

-- Number of reference frames, should be larger than 4 for recon preview,
-- exactly 4 for IP encode
NUM_AVC_REFERENCES = 6;

-- Renderer field order.  This should match the field order of the input.
-- For example, DVD input is typically BFF.
RENDER_BFF 	= 1;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

BITSTREAM_OUTPUT =  1;
ACKINTERVAL = 8;

sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=1, videocontrol=0x00101000, interleave=1});
sysctl0:setVPortMode({port=1, live=0});

board.setVideoInput({port=0, format=FORMAT, mode=string.format("%s",VIN)});
board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});

sysctl0:setVerboseLevel({param=1});

CROPWIDTH=720;
CROPHEIGHT=480;
COMPWIDTH=OUTGRIDW*OUTW;
COMPHEIGHT=OUTGRIDH*OUTH;
COMPYOFFSET= (480-(OUTGRIDH*OUTH))/2;

avoc0 = sysctl0:avocCreate({name="avoc0", maxWidth=COMPWIDTH, maxHeight=COMPHEIGHT});
avoc0:setOutput480i();
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_X, value=0});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_Y, value=COMPYOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_ENABLE, value=1});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_BFF, value=RENDER_BFF});
avoc0:setParam({param=Q_AVOC_CMP_VPU_BG_COLOR, value=0x10808000});
avoc0:activateCfg();

ENCCH = ENCCH - 1;
OUTCH = OUTCH - 1;

lvpp0 = sysctl0:lvppCreate({name="lvpp0", slot=0, port=0, maxWidth=720, maxHeight=480, numVidFrame=4});

ws = sysctl0:createWriteStreamer({name="ws0"});
ws:showrate({enable=SHOWRATE});
ws:start()

lvpp0:crop({cropWidth=720, cropHeight=480, vertOffset=0, horzOffset=0});
lvpp0:size({outputWidth=720, outputHeight=480});
lvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=263});
lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=0});
lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=22});
lvpp0:setParam({param=Q_LVPP_CMP_DEST_PROG, value=0});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=3003});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=3003});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=8});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=9});
lvpp0:activateCfg();

-- CREATE ENCODERS
nvppenc = {};
avcenc = {};
avsnenc = {};
videoStreamId = 0;
x = 0;
y = 0;
for i = 0,ENCCH do
  nvppenc[i] = sysctl0:nvppCreate({name="nvpp"..i, maxWidth=ENCW, maxHeight=ENCH, maxPMEDSFactor=1});
  avcenc[i] = sysctl0:avcencCreate({name="avcenc"..i, ws=ws, maxwidth=ENCW, maxheight=ENCH, numreferenceframes=NUM_AVC_REFERENCES});
  avsnenc[i] = sysctl0:avsnCreate({name="avsn"..i});
  videoStreamId = i + 2;

  -- CONFIGURE VPP
  -- Setup_VPP(vpp_instance, in_width, in_height, is_interlaced, is_420, pme_hscale, pme_vscale)
  Setup_VPP(nvppenc[i], ENCW, ENCH, 0, 0, 2, 2)
  nvppenc[i]:setParam({param=Q_NVPP_CMP_CROP_OFFSET_X,        value=x});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_CROP_OFFSET_Y,        value=y});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_PIXEL_AR_X,           value=8});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_PIXEL_AR_Y,           value=9});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_PMEDS_H,              value=2});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_PMEDS_V,              value=2});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_STC_PER_FRAME,        value=3003});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_STC_PER_OUTPUT_FRAME, value=3003});
  -- We have to set this to make vlc work to enforce a true 352x240 image with
  -- no crop rect.  If we had not set this then we would actually create a 
  -- 352x256 with a 352x240 crop rect which crashes VLC but is otherwise
  -- correct.  This parameter prevents any padding, even to a multiple of 16
  -- for AVC encoding.  It also inhibits padding to 32 for PME operation 
  -- but since we know we are aligned and a proper size we can do this
  nvppenc[i]:setParam({param=Q_LVPP_CMP_SCALE_MB_ALIGN,       value=0});
  nvppenc[i]:activateCfg();

  -- calculate crop offsets 
  x = x + (CROPWIDTH / ENCGRIDW);
  if (x == CROPWIDTH) then
    x = 0;
    y = y + (CROPHEIGHT / ENCGRIDH);
  end

  -- AVC SETUP

  -- we use the high quality configuration since we have enough performance
  avcenc_480i_highquality(avcenc[i], "high", 31, BITRATE[i], Q_AVCENC_CFP_GOP_STRUCTURE_IP, "frame")
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, value=3003});
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT, value=BITSTREAM_OUTPUT});
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_CLOSED_GOP, value=1});
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_NEW_SPS_EACH_IDR, value=1});
  if PREVIEW == 2 then
    avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_ENCODER_PREVIEW, value=1});
  end
  avcenc[i]:activateVideoEncCfg();

  -- Connect LVPP->NVPP->AVCENC
  nvppenc[i]:bindInput({src=lvpp0});
  avcenc[i]:bindVideoInput({src=nvppenc[i]});

  if (STREAMING == 1) then
    ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=1,max=%d", 
                HOST, PORT+i,MAXPACKETSIZE), sid = {videoStreamId}});
  elseif (STREAMING == 2) then
      ws:open({type="rtp",
               file=string.format("sip=%s,prt=%d,ses=stream%d,vid=h264,mod=0,msz=%d",
               SERVER, PORT, i, MAXPACKETSIZE), sid= {videoStreamId}});
  end
  if (STORAGE == 1) then
      ws:open({type="file", file=OutputFileName[i]..".qbx", sid = {videoStreamId}});
  end
  avcenc[i]:setAckInterval({interval=ACKINTERVAL});
  avcenc[i]:setStreamID({streamid=videoStreamId});
end

-- Set up the video channels for the live output 
vch = {};
x = 0;
y = 0;
for i = 0,ENCCH do
  vch[i] = avoc0:addVideoChannel();

  if PREVIEW == 1 then
    avsnenc[i]:bindVideoInput({src=nvppenc[i]});
  elseif PREVIEW == 2 then
    avsnenc[i]:bindVideoInput({src=avcenc[i]});
    avsnenc[i]:setParam({param=Q_AVSN_CMP_EXTERNAL_STC_OFFSET, value=-15015});
    avsnenc[i]:activateCfg();
  end

  avoc0:setVideoChannelSource({ch=vch[i], src=avsnenc[i]});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=ENCW});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=ENCH});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=OUTW});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=OUTH});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=x});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=y});

  x = x + OUTW;
  if x == COMPWIDTH then
    x = 0;
    y = y + OUTH;
  end
end
avoc0:activateVideoChannelCfg();

-- START
avoc0:start();

for i = 0,OUTCH do
  avsnenc[i]:start();
end

for i = 0,ENCCH do
  nvppenc[i]:start();
  avcenc[i]:record();
end

lvpp0:start();

function done()
  lvpp0:flush();

  waitForEvents({objid=avcenc[0], timeout=1000000, any=0, events={Q_AVCENC_EV_BITSTREAM_FLUSHED}});

  os.execute("echo flushed");

  for i = 0,ENCCH do
    if (STREAMING == 1) then
      ws:close({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=1", HOST, PORT+i)});
    elseif (STREAMING == 2) then
    end
  end
end

