--[[

This script used to encode 4 CIF resolution streams from the NTSC output
of the TW2835.  The TW2835 scales and composes 4 CIF images into a single
D1 image, which in then is divided into 4 streams which are each encoded
and streamed.  

Splitting of the single stream into four separate video inputs is done
using four NVPP objects each of which is connected to a single LVPP
object handling video port 0.  The display consists of a 2x2 grid of
either live or reconstructed preview that is similar to that of the 
TW2835 but can have an OSD overlay.

Streaming is done either with raw UDP packets or with RTP.

--]]

----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

BITRATE 	= BITRATE or {};

-- Number of live encoded channels
ENCCH		= ENCCH or 4;

-- Number of decoded channels
DECCH       = DECCH or 4;

-- Host IP and base port for udp live stream (port, port+1, port+2...)
-- Streaming protocol 0 for nothing, 1 for UDP, 2 for RTP
STREAMING 	= STREAMING or 2;
SERVER      = SERVER or "10.10.10.207";
HOST 		= HOST or "192.168.40.38";
PORT 		= PORT or 554;
MAXPACKETSIZE = MAXPACKETSIZE or 1472;

-- show or hide bitrate 0 for off, 1 for on
SHOWRATE 	= SHOWRATE or 0;

-- video input mode svideo or composite
VIN 		= VIN or "svideo";
VOUT		= VOUT or "svideo";
VIDEO_FORMAT= VIDEO_FORMAT or "480i"

-- Width and height of each live encoded stream
ENCW 		= ENCW or 352;
ENCH 		= ENCH or 240;

-- Transfer size per encoded
MAIN_AVCENC_XFER_SIZE = MAIN_AVCENC_XFER_SIZE or 4;

-- Geometry of input grid (typically 2x2 for TW2835)
ENCGRIDW 	= ENCGRIDW or 2;
ENCGRIDH 	= ENCGRIDH or 2;

-- Bitrate of each live encoded stream
BITRATE[0] 	= BITRATE[0] or 500000;
BITRATE[1] 	= BITRATE[1] or 500000;
BITRATE[2] 	= BITRATE[2] or 500000;
BITRATE[3] 	= BITRATE[3] or 500000;

-- Output filenames for encoders
OutputFileName = {};
OutputFileName[0] = "/media/sda1/vid4/dvr_test0";
OutputFileName[1] = "/media/sda1/vid4/dvr_test1";
OutputFileName[2] = "/media/sda1/vid4/dvr_test2";
OutputFileName[3] = "/media/sda1/vid4/dvr_test3";

-- Width and height of each decoded stream
DECW        = DECW or 352;
DECH 		= DECH or 240;

-- Output filenames for decoders
InputFileName = {};
InputFileName[0] = "/media/sda1/vid4/dvr_test0";
InputFileName[1] = "/media/sda1/vid4/dvr_test1";
InputFileName[2] = "/media/sda1/vid4/dvr_test2";
InputFileName[3] = "/media/sda1/vid4/dvr_test3";

-- Number of rendered images and their scaled size
OUTCH 		= OUTCH or 4;
OUTW 		= OUTW or 352;
OUTH 		= OUTH or 240;

-- Geometry of output grid (typically 2x2 for TW2835)
OUTGRIDW 	= OUTGRIDW or 2;
OUTGRIDH 	= OUTGRIDH or 2;

-- Preview mode (1 for live, 2 for reconstructed, 0 for off)
PREVIEW 	= PREVIEW or 1;

-- Number of reference frames, should be larger than 4 for recon preview,
-- exactly 4 for IP encode
NUM_AVC_REFERENCES = 6;

-- Renderer field order.  This should match the field order of the input.
-- For example, DVD input is typically BFF.
RENDER_BFF 	= 1;

-- Enable OSD
ENABLE_OSD = ENABLE_OSD or 0;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

BITSTREAM_OUTPUT =  1;
ACKINTERVAL = 8;

sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=1, videocontrol=0x00101000, interleave=1});
sysctl0:setVPortMode({port=1, live=0});

board.setVideoInput({port=0, format=VIDEO_FORMAT, mode=string.format("%s",VIN)});
board.setVideoOutput({port=0, format=VIDEO_FORMAT, mode=VOUT});

sysctl0:setVerboseLevel({param=1});

CROPWIDTH=720;
CROPHEIGHT=480;
COMPWIDTH=OUTGRIDW*OUTW;
COMPHEIGHT=OUTGRIDH*OUTH;
COMPYOFFSET= (480-(OUTGRIDH*OUTH))/2;

--[[

    ** enable osd if ENABLE_OSD is set. 
    ** osdSrcWidth is the physical layout of memory. In our case
        it must be set to 2048 to deal with our x/y mapping.
    ** bppFormat is set to ARGB4444, 16bpp, which requires two
        bytes per pixel and that's why gfxSize is multiplied by 2.
--]]

if ENABLE_OSD == 0 then
    avoc0 = sysctl0:avocCreate({name="avoc0", maxWidth=COMPWIDTH, maxHeight=COMPHEIGHT});
else 

    local osdWidth      = CROPWIDTH;
    local osdSrcWidth   = 2048;
    local osdHeight     = CROPHEIGHT;
    local osdSrcHeight  = CROPHEIGHT;
    local osdFormat     = 0x11;
    local Bpp           = 1;
    local gfxSize       = (osdSrcWidth * osdSrcHeight * 2);

    if osdFormat ~= 0x06 then
        Bpp = 2;
        gfxSize = gfxSize * Bpp;
    end

    avoc0 = sysctl0:avocCreate(
        {name="avoc0", maxWidth=COMPWIDTH, maxHeight=COMPHEIGHT, gfx0Size=gfxSize, allocGfx=1});

    osdInit(avoc0, osdSrcWidth, osdSrcHeight, osdWidth, osdHeight, osdFormat);

end

avoc0:setOutput480i();
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_X, value=0});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_Y, value=COMPYOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_ENABLE, value=1});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_BFF, value=RENDER_BFF});
avoc0:setParam({param=Q_AVOC_CMP_VPU_BG_COLOR, value=0x10808000});
avoc0:activateCfg();

ENCCH = ENCCH - 1;
DECCH = DECCH - 1;
OUTCH = OUTCH - 1;

lvpp0 = sysctl0:lvppCreate({name="lvpp0", slot=0, port=0, maxWidth=720, maxHeight=480, numVidFrame=4});

ws = sysctl0:createWriteStreamer({name="ws0", queuesize=500});
ws:showrate({enable=SHOWRATE});
ws:start()

lvpp0:crop({cropWidth=720, cropHeight=480, horzOffset=0, vertOffset=0});
lvpp0:size({outputWidth=720, outputHeight=480});
lvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=263});
lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=0});
lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=22});
lvpp0:setParam({param=Q_LVPP_CMP_DEST_PROG, value=0});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=3003});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=3003});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=8});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=9});
lvpp0:activateCfg();

-------------------- Create Encoders
nvppenc = {};
avcenc = {};
videoStreamId = 0;
x = 0;
y = 0;
for i = 0,ENCCH do
  nvppenc[i] = sysctl0:nvppCreate({name="nvpp"..i, maxWidth=ENCW, maxHeight=ENCH, maxPMEDSFactor=1});
  avcenc[i] = sysctl0:avcencCreate({name="avcenc"..i, ws=ws, maxwidth=ENCW, maxheight=ENCH, numreferenceframes=NUM_AVC_REFERENCES});
  videoStreamId = i + 2;

  -- CONFIGURE VPP
  -- Setup_VPP(vpp_instance, in_width, in_height, is_interlaced, is_420, pme_hscale, pme_vscale)
  Setup_VPP(nvppenc[i], ENCW, ENCH, 0, 0, 2, 2)
  nvppenc[i]:setParam({param=Q_NVPP_CMP_CROP_OFFSET_X,        value=x});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_CROP_OFFSET_Y,        value=y});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_PIXEL_AR_X,           value=8});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_PIXEL_AR_Y,           value=9});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_PMEDS_H,              value=2});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_PMEDS_V,              value=2});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_STC_PER_FRAME,        value=3003});
  nvppenc[i]:setParam({param=Q_NVPP_CMP_STC_PER_OUTPUT_FRAME, value=3003});
  -- We have to set this to make vlc work to enforce a true 352x240 image with
  -- no crop rect.  If we had not set this then we would actually create a 
  -- 352x256 with a 352x240 crop rect which crashes VLC but is otherwise
  -- correct.  This parameter prevents any padding, even to a multiple of 16
  -- for AVC encoding.  It also inhibits padding to 32 for PME operation 
  -- but since we know we are aligned and a proper size we can do this
  nvppenc[i]:setParam({param=Q_LVPP_CMP_SCALE_MB_ALIGN,       value=0});
  nvppenc[i]:activateCfg();

  -- calculate crop offsets 
  x = x + (CROPWIDTH / ENCGRIDW);
  if x == CROPWIDTH then
    x = 0;
    y = y + (CROPHEIGHT / ENCGRIDH);
  end

  -- we use the high quality configuration since we have enough performance
  avcenc_480i_highquality(avcenc[i], "high", 31, BITRATE[i], Q_AVCENC_CFP_GOP_STRUCTURE_IP, "frame")
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, value=3003});
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT, value=BITSTREAM_OUTPUT});
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_CLOSED_GOP, value=1});
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_NEW_SPS_EACH_IDR, value=1});
  if PREVIEW == 2 then
    avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_ENCODER_PREVIEW, value=1});
  end
  avcenc[i]:setTransferSize({size=MAIN_AVCENC_XFER_SIZE});
  avcenc[i]:activateVideoEncCfg();

  -- Connect LVPP->NVPP->AVCENC
  nvppenc[i]:bindInput({src=lvpp0});
  avcenc[i]:bindVideoInput({src=nvppenc[i]});

  if (STREAMING == 1) then
    ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=1,max=%d", 
                HOST, PORT+i,MAXPACKETSIZE), sid = {videoStreamId}});
  elseif (STREAMING == 2) then
      ws:open({type="rtp",
               file=string.format("sip=%s,prt=%d,ses=stream%d,vid=h264,mod=0,msz=%d",
               SERVER, PORT, i, MAXPACKETSIZE), sid= {videoStreamId}});
  end
  avcenc[i]:setAckInterval({interval=ACKINTERVAL});
  avcenc[i]:setStreamID({streamid=videoStreamId});
end

-------------------- Set up output for live display
evch = {};
avsnenc = {};
EncRenderDisable = {};

-- used to remember x/y of each channel
EncChannelX = {};
EncChannelY = {};

x = 0;
y = 0;

for i = 0,ENCCH do
  avsnenc[i] = sysctl0:avsnCreate({name="avsn"..i});
  evch[i] = avoc0:addVideoChannel();

  if PREVIEW == 1 then
    avsnenc[i]:bindVideoInput({src=nvppenc[i]});
  elseif PREVIEW == 2 then
    avsnenc[i]:bindVideoInput({src=avcenc[i]});
    avsnenc[i]:setParam({param=Q_AVSN_CMP_EXTERNAL_STC_OFFSET, value=-15015});
    avsnenc[i]:activateCfg();
  end

  avoc0:avsnConnect({src=avsnenc[i]});
  avoc0:setVideoChannelSource({ch=evch[i], src=avsnenc[i]});
  avoc0:setVideoChannelParam({ch=evch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=ENCW});
  avoc0:setVideoChannelParam({ch=evch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=ENCH});
  avoc0:setVideoChannelParam({ch=evch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=evch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
  avoc0:setVideoChannelParam({ch=evch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=OUTW});
  avoc0:setVideoChannelParam({ch=evch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=OUTH});
  avoc0:setVideoChannelParam({ch=evch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=x});
  avoc0:setVideoChannelParam({ch=evch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=y});

  -- remember that encoders are all enabled on the display
  EncRenderDisable[i] = 0;

  -- Remember x, y
  EncChannelX[i] = x;
  EncChannelY[i] = y;

  x = x + OUTW;
  if x == COMPWIDTH then
    x = 0;
    y = y + OUTH;
  end
end

avoc0:activateVideoChannelCfg();

---------  Create Decoders

-- Allocate compressed video buffers for the decoders.
videoInputBufferSize = 250 * 2048;
audioInputBufferSize = 50 * 2048;
sysctl0:configure({param=Q_SYS_CFG_DECODER_INPUT_PARTITION_SIZE,
                   value=4*videoInputBufferSize+
                         4*audioInputBufferSize});

avdec = {};
avsndec = {};

for i = 0,DECCH do
    local audioStreamId = 2*i + 1;
    local videoStreamId = 2*i + 2;

    avdec[i] = sysctl0:avdecCreate({name="avdec" ..i, atype=Q_AVDEC_CFP_AUDIO_CODEC_QMA, 
                                    vtype=Q_AVDEC_CFP_VIDEO_CODEC_AVC, 
                                    audioBufferSize=audioInputBufferSize, videoBufferSize=videoInputBufferSize, 
                                    maxWidth=352, maxHeight=240, numoutputframes=6, videoAUs=1024});
    avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_STREAM_ID, value=videoStreamId});
    avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_ID, value=audioStreamId});
    avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_RATE, value=24000});
    avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_SIZE, value=16});
    avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, value=Q_AVDEC_CFP_AUDIO_CHANNEL_STEREO});
    avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_OVERRIDE, value=Q_AVDEC_CFP_TRUE});
    avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_DEFAULT, value=3003});
    -- must set this to match decoder buffer size
    avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_UNDERFLOW_THRESHOLD, value=3*videoInputBufferSize/4});
    avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_UNDERFLOW_AU_THRESHOLD, value=768});
    avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_UNDERFLOW_MARGIN, value=10000});
    avdec[i]:activateCfg();
end

--------------- Create decoder display
dvch = {};
avsndec = {};
DecRenderDisable = {};

-- used to remember x/y of each channel
DecChannelX = {};
DecChannelY = {};

x = 0;
y = 0;

for i = 0,DECCH do
  dvch[i] = avoc0:addVideoChannel();
  avsndec[i] = sysctl0:avsnCreate({name="avsndec"..i});

  -- bind to the matching decoder
  avsndec[i]:bindVideoInput({src=avdec[i]});

  -- By default we disable decoder channels until playback starts
  avoc0:avsnConnect({src=avsndec[i]});
  avoc0:setVideoChannelSource({ch=dvch[i], src=0});
  avoc0:setVideoChannelParam({ch=dvch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=DECW});
  avoc0:setVideoChannelParam({ch=dvch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=DECH});
  avoc0:setVideoChannelParam({ch=dvch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=dvch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
  avoc0:setVideoChannelParam({ch=dvch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=OUTW});
  avoc0:setVideoChannelParam({ch=dvch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=OUTH});
  avoc0:setVideoChannelParam({ch=dvch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=x});
  avoc0:setVideoChannelParam({ch=dvch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=y});

  DecChannelX[i] = x;
  DecChannelY[i] = y;

  -- remember that decoders are all disabled on the display
  DecRenderDisable[i] = 1;

  x = x + OUTW;
  if x == COMPWIDTH then
    x = 0;
    y = y + OUTH;
  end
end

-- Create readstreamer
bsr = {};
rs0 = sysctl0:createReadStreamer({name="rs0"});

--------------- START
avoc0:start();

for i = 0,ENCCH do
  avsnenc[i]:start();
  nvppenc[i]:start();
  avcenc[i]:record();
end

lvpp0:start();

function showosd()
  if ENABLE_OSD == 1 then
    avoc0:setOSDParam({plane=0, param=Q_AVOC_CMP_OSD_SOURCE_FRAME, value=avoc0.gfx0Addr});
    avoc0:activateOSDCfg();
  end
end

function hideosd()
  if ENABLE_OSD == 1 then
    avoc0:setOSDParam({plane=0, param=Q_AVOC_CMP_OSD_SOURCE_FRAME, value=0});
    avoc0:activateOSDCfg();
  end
end

function showlive(i)
  avoc0:setVideoChannelSource({ch=evch[i], src=avsnenc[i]});
  avoc0:activateVideoChannelCfg();

  -- remember state
  EncRenderDisable[i] = 0;
end

function hidelive(i)
  avoc0:setVideoChannelSource({ch=evch[i], src=0});
  avoc0:activateVideoChannelCfg();

  -- remember state
  EncRenderDisable[i] = 1;
end

function startrecord(i)
  local videoStreamId = i + 2;
  ws:open({type="file", file=OutputFileName[i]..".qbx", sid = {videoStreamId}});
end

function startrecordall()
  ws:open({type="file", file=OutputFileName[0]..".qbx", sid = {0}});
end

function stoprecord(i)
  local videoStreamId = i + 2;
  ws:close({type="file", file=OutputFileName[i]..".qbx"});
end

function startplayback(i)
  local encId = i + 2;
  local decId = 2*i + 2;

  rs0:bindDecoder({dec=avdec[i]});
  bsr[i] = rs0:open({type="qbox", file=InputFileName[i]..".qbx", srcId={encId}, codecId={decId}});

  -- Start decoder
  avsndec[i]:start();
  avdec[i]:start();
  rs0:play({bsr=bsr[i]});
end

function startplaybackall(ch)
  local eId = {};
  local dId = {};
  
  -- Encode/Decode Stream IDs (need to match AVCENC and AVDEC, respectively)
  for i=0,ch-1 do
    eId[i] = i + 2;
    dId[i] = 2*i + 2;
  end
  
  bsr[0] = rs0:open({type="qbox", file=InputFileName[0]..".qbx", srcId=eId, codecId=dId});
  
  for i=0,ch-1 do
    rs0:bindDecoder({dec=avdec[i]});
    avsndec[i]:start();
    avdec[i]:start();
  end
  
  rs0:play({bsr=bsr[0]});
end

function stopplayback(i)
  rs0:close({bsr=bsr[i]});
  avdec[i]:stop();
  avdec[i]:waitForVideoEOS()
  avsndec[i]:stop()
  rs0:unbindDecoder({dec=avdec[i]});
end

function stopplaybackall(i)
  rs0:close({bsr=bsr[0]});
  
  for i=0,ch-1 do
    avdec[i]:stop();
    avdec[i]:waitForVideoEOS()
    avsndec[i]:stop()
    rs0:unbindDecoder({dec=avdec[i]});
  end
end

function showplayback(i)
  avoc0:setVideoChannelSource({ch=dvch[i], src=avsndec[i]});
  avoc0:activateVideoChannelCfg();

  -- remember state
  DecRenderDisable[i] = 0;
end

function hideplayback(i)
  avoc0:setVideoChannelSource({ch=dvch[i], src=0});
  avoc0:activateVideoChannelCfg();

  -- remember state
  DecRenderDisable[i] = 1;
end

function showfullscreenplayback(i)

  -- turn off all decoders for now
  for i = 0,DECCH do
    avoc0:setVideoChannelSource({ch=dvch[i], src=0});
  end

  -- turn off all encoders for now
  for i = 0,ENCCH do
    avoc0:setVideoChannelSource({ch=evch[i], src=0});
  end

  avoc0:activateVideoChannelCfg();

  -- make the selected channel full size
  avoc0:setVideoChannelSource({ch=dvch[i], src=avsndec[i]});
  avoc0:setVideoChannelParam({ch=dvch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=DECW});
  avoc0:setVideoChannelParam({ch=dvch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=DECH});
  avoc0:setVideoChannelParam({ch=dvch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=dvch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
  avoc0:setVideoChannelParam({ch=dvch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=OUTW*OUTGRIDW});
  avoc0:setVideoChannelParam({ch=dvch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=OUTH*OUTGRIDH});
  avoc0:setVideoChannelParam({ch=dvch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=dvch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=0});
  avoc0:activateVideoChannelCfg();

  -- Remember which channel is full screen
  FS_CHANNEL = i;

end

function hidefullscreenplayback()

  -- turn off full screen
  avoc0:setVideoChannelSource({ch=dvch[FS_CHANNEL], src=0});
  avoc0:activateVideoChannelCfg();

  -- Put the full screen channel back to the right size
  avoc0:setVideoChannelParam({ch=dvch[FS_CHANNEL], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=DECW});
  avoc0:setVideoChannelParam({ch=dvch[FS_CHANNEL], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=DECH});
  avoc0:setVideoChannelParam({ch=dvch[FS_CHANNEL], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=dvch[FS_CHANNEL], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
  avoc0:setVideoChannelParam({ch=dvch[FS_CHANNEL], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=OUTW});
  avoc0:setVideoChannelParam({ch=dvch[FS_CHANNEL], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=OUTH});
  avoc0:setVideoChannelParam({ch=dvch[FS_CHANNEL], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, 
                              value=DecChannelX[FS_CHANNEL]});
  avoc0:setVideoChannelParam({ch=dvch[FS_CHANNEL], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, 
                              value=DecChannelY[FS_CHANNEL]});
  avoc0:activateVideoChannelCfg();

  -- restore decoders
  for i = 0,DECCH do
    if DecRenderDisable[i] == 0 then
      avoc0:setVideoChannelSource({ch=dvch[i], src=avsndec[i]});
    else
      avoc0:setVideoChannelSource({ch=dvch[i], src=0});
    end
  end

  -- restore encoders
  for i = 0,ENCCH do
    if EncRenderDisable[i] == 0 then
      avoc0:setVideoChannelSource({ch=evch[i], src=avsnenc[i]});
    else
      avoc0:setVideoChannelSource({ch=evch[i], src=0});
    end
  end

  avoc0:activateVideoChannelCfg();
end

-- All streams will be written to one file if set to 1
WRITE_ONE_FILE = 0;

function demo(ch)

  if (ch == nil) then
    print "must specify channels\n";
    return
  end

  -- start recording
  if (WRITE_ONE_FILE == 0) then
    for i=0,ch-1 do
      print("Start recording channel "..i);
      startrecord(i);
    end
  else
    print("Start recording ALL channels");
    startrecordall();
  end

  print("Sleeping for 15 seconds");
  os.execute("sleep 15");

  for i=0,ch-1 do
    print("Hiding live display of channel "..i);
    hidelive(i);

    print("Displaying decoder for channel "..i);
    showplayback(i);

    if (WRITE_ONE_FILE == 0) then
      print("Starting playback for channel "..i);
      startplayback(i);
    end
  end
  
  if (WRITE_ONE_FILE == 1) then
    print("Starting playback for ALL channels");
    startplaybackall(ch);
  end

end

