--[[

This script is used to test simultaneous encode/decode.  The stream
that is decoded is looped back in the host via a named pipe that
is created in this script.

The script can optionally create a side-by-side display of the
decoded and live streams for a delay demonstration.

--]]


----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

-- Video input mode
-- "hdmi" HDMI input and output
-- "component"
-- "svideo"
-- "composite"
VIN 		= VIN or "component";

-- Video output mode
-- "hdmi" HDMI output (1080i and 720p only)
-- "svideo" SVIDEO input/output (note composite is driven at same time on output)
VOUT		= VOUT or "component";

-- Video format (only 480i and 720p for now)
-- "1080i"
-- "720p"
-- "480i"
FORMAT 		= FORMAT or "720p";

-- Video bitrate
VBITRATE 	=  VBITRATE or 3000000;

VIDEO_ONLY = VIDEO_ONLY or 1;

-- Controls the audio encoder type
--   "AAC-LC"
--   "ER-AAC-LD"
--   "MPEG-LAYER-2"
--   "G711-A-LAW"
--   "G711-MU-LAW"
--   "G711" (defaults to mu-law)
--   "G722"
--   "G726"
--   "G728"
AUDIO_ENCODER = AUDIO_ENCODER or "G711";

-- Audio sample rate (may be overridden depending on codec)
SAMPLERATE = SAMPLERATE or 32000;

-- Audio stereo/mono (may be overridden depending on codec)
AUDIO_STEREO = AUDIO_STEREO or 0;

-- Audio bitrate (may be overridden depending on codec)
if(AUDIO_STEREO) then
   ABITRATE = ABITRATE or 64000;
else
   ABITRATE = ABITRATE or 56000;
end

-- Audio frame size (may be overridden depending on codec)
AUDIO_FRAME_SIZE = AUDIO_FRAME_SIZE or 256;

-- Debug variable to disable bitstream output to host
BITSTREAM_OUTPUT =  BITSTREAM_OUTPUT or 1;

-- Set Target Latency Values
TARGET_LATENCY_TOLERANCE = TARGET_LATENCY_TOLERANCE or 0; -- in milliseconds
TARGET_LATENCY_UNDERFLOW = TARGET_LATENCY_UNDERFLOW or 0;  -- in milliseconds
TARGET_LATENCY_OVERFLOW  = TARGET_LATENCY_OVERFLOW or 0;   -- in milliseconds

-- Controls the GOP structure
-- "IBBRBP" : Pyramidal GOP structure
-- "IP" : No B frames
GOP_STRUCTURE =  GOP_STRUCTURE or "IP";

-- Controls the coding structure of a frame or field pair
-- "mbaff" : mb adaptive field/frame
-- "field" : field coding
-- "frame" : frame coding.
-- Note progressive inputs are forced to frame coding
FRAME_STRUCTURE =  FRAME_STRUCTURE or "frame";

-- we override specified filename here as we are using
-- a named pipe
FILENAME 	= "/tmp/loopback.qbx";

FRAMERATE 	= 30;

MAXNALUEN 	= MAXNALUEN or 0;

MAX_NALU_LENGTH = MAX_NALU_LENGTH or 1400;

VIDEO_AUS 	= 2048;

COMPOSEDISPLAY = 1;

INPUTBFF 	= 1;

HOST 		= HOST or "127.0.0.1";
PORT 		= PORT or 9700;
MAX_PACKET_SIZE = MAX_PACKET_SIZE or 768;

-- Set this flag to 1 or 2 if you want raw udp or rtp streaming enabled.
-- Set to 0 if you want local loopback using a named pipe.
STREAMING	= STREAMING or 0;

-- RTP/RTSP Settings
-- ip address of rtp server.
SERVER_IP = SERVER_IP or "10.32.84.176"
-- port of rtp server
SERVER_PORT = SERVER_PORT or 554
-- session identifier for rtp server
SESSION_ID = SESSION_ID or "test";
-- rtp server mode
SERVER_MODE = SERVER_MODE or 0;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

NUM_AVC_REFERENCES = NUM_AVC_REFERENCES or 4;
NUM_AVDEC_REFERENCES = NUM_AVDEC_REFERENCES or 10;
AVDEC_BUFFER_SIZE = AVDEC_BUFFER_SIZE or 8000000;
ENC_OUTPUT_BUFFER = 4000000;

VIDEOSTREAMID = 2;
AUDIOSTREAMID = 1;

if (FORMAT == "720p") then
  HSIZE = 1280;
  VSIZE = 720;
  FRAME_STRUCTURE = "frame"; -- override frame structure
  MAX_PME_DOWNSCALE = 4;
  NUM_VIN_FRAMES = 11;
  TF_LUMA_BYPASS = 0;
  TF_CHROMA_BYPASS = 1;
  NATIVE_FRAME_TIME = 1500
  FRAME_TIME = 90000/FRAMERATE;
  INPUT_PIXEL_AR_X = 1;
  INPUT_PIXEL_AR_Y = 1;
  COMPWIDTH = 1280;
  COMPHEIGHT = 720;
  COMPRATEDIVIDER = 2;
elseif (FORMAT == "1080i") then
  HSIZE = 1920;
  VSIZE = 1080;
  FRAME_STRUCTURE = "mbaff"; -- override frame structure
  MAX_PME_DOWNSCALE = 4;
  NUM_VIN_FRAMES = 9;
  TF_LUMA_BYPASS = 0;
  TF_CHROMA_BYPASS = 1;
  NATIVE_FRAME_TIME = 3003
  FRAME_TIME = 90000/FRAMERATE;
  INPUT_PIXEL_AR_X = 1;
  INPUT_PIXEL_AR_Y = 1;
  COMPWIDTH = 1920;
  COMPHEIGHT = 1080;
  COMPRATEDIVIDER = 1;
  if (VIN == "component") then
        HSTART_PIXEL = 235;
  else
        HSTART_PIXEL = 193;
  end

  -- performance/memory optimization needed for this to work: 16 CIF
  HSIZE = 1440;
  VSIZE = 960;
  NUM_AVDEC_REFERENCES = 7;
--  AVDEC_BUFFER_SIZE = 3000000;
--  ENC_OUTPUT_BUFFER = 4000000;
  COMPOSEDISPLAY = 0;

elseif (FORMAT == "480i") then
  HSIZE = 720;
  VSIZE = 480;
  MAX_PME_DOWNSCALE = 1;
  NUM_VIN_FRAMES = 11;
  TF_LUMA_BYPASS = 0;
  TF_CHROMA_BYPASS = 0;
  NATIVE_FRAME_TIME = 3003;
  FRAME_TIME = 90090/FRAMERATE;
  INPUT_PIXEL_AR_X = 8;
  INPUT_PIXEL_AR_Y = 9;
  COMPWIDTH = 720;
  COMPHEIGHT = 480;
  COMPRATEDIVIDER = 1;
else
  error("Invalid format " .. tostring(FORMAT))
end


-- always render from top left
COMPXOFFSET = 0;
COMPYOFFSET = 0;

if (GOP_STRUCTURE == "IP") then
  gopstruct = Q_AVCENC_CFP_GOP_STRUCTURE_IP;
elseif  (GOP_STRUCTURE == "IBBRBP") then
  gopstruct = Q_AVCENC_CFP_GOP_STRUCTURE_IBBRBP;
else
  print("Invalid GOP structure " .. GOP_STRUCTURE .. "\n");
end

-- modulo 16 size
VSIZE16 = 16*math.ceil(VSIZE/16)


vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end

if (FORMAT == "1080i") then
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT, sync=vout_sync});
  sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=0, videocontrol=0x00000ba0, interleave=1});
elseif (FORMAT == "720p") then
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
  sysctl0:setVPortMode({port=0, live=1, interlaced=0, bottomfieldfirst=0, videocontrol=0x00000ba8, interleave=1});
elseif (FORMAT == "480i") then
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
  sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=0, videocontrol=0x00101000, interleave=1});
end

-- force non-live mode for compose
sysctl0:setVPortMode({port=1, live=0});

board.setVideoInput({port=0, format=FORMAT, mode=VIN});

-- audio parameters and board setup for audio
if (VIDEO_ONLY == 0) then
   -- setup audio parameters with over-rides as necessary
   QAC_AOT = 2;
   Q711_LAW = Q_AUDENC_CFP_Q711_U_LAW;
   if (AUDIO_ENCODER == "AAC-LC") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_QAC;
      AUDIO_FRAME_SIZE = 1024;
      QAC_AOT = 2;
   elseif (AUDIO_ENCODER == "ER-AAC-LD") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_QAC;
      AUDIO_FRAME_SIZE = 512;
      QAC_AOT = 23;
   elseif (AUDIO_ENCODER == "MPEG-LAYER-2") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_QMA;
      AUDIO_FRAME_SIZE = 1152;
   elseif (AUDIO_ENCODER == "G711-A-LAW") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q711;
      Q711_LAW = Q_AUDENC_CFP_Q711_A_LAW;
      -- AUDIO_FRAME_SIZE is flexible
      SAMPLERATE = 8000;
      AUDIO_STEREO = 0;
      ABITRATE = 64000;
   elseif (AUDIO_ENCODER == "G711-MU-LAW") or (AUDIO_ENCODER == "G711") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q711;
      Q711_LAW = Q_AUDENC_CFP_Q711_U_LAW;
      -- AUDIO_FRAME_SIZE is flexible
      SAMPLERATE = 8000;
      AUDIO_STEREO = 0;
      ABITRATE = 64000;
   elseif (AUDIO_ENCODER == "G722") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q722;
      -- AUDIO_FRAME_SIZE is flexible
      SAMPLERATE = 16000;
      AUDIO_STEREO = 0;
      ABITRATE = 64000;
   elseif (AUDIO_ENCODER == "G726") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q726;
      -- AUDIO_FRAME_SIZE is flexible
      SAMPLERATE = 8000;
      AUDIO_STEREO = 0;
      if (ABITRATE ~= 40000) and (ABITRATE ~= 32000) and (ABITRATE ~= 24000) and (ABITRATE ~= 16000) then
         ABITRATE = 40000; -- set usable default
      end
   elseif (AUDIO_ENCODER == "G728") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q728;
      -- AUDIO_FRAME_SIZE is flexible
      SAMPLERATE = 8000;
      AUDIO_STEREO = 0;
      ABITRATE = 16000;
   else
      error("Invalid audio encoder " .. tostring(AUDIO_ENCODER))
   end
   board.setAudioSampleFreq({port=0, freq=SAMPLERATE});
end

ws = sysctl0:createWriteStreamer({name="ws0"});
ws:start()

lvpp0 = sysctl0:lvppCreate({name="lvpp0", slot=0, port=0, maxWidth=HSIZE, maxHeight=VSIZE16,
                            numVidFrame=NUM_VIN_FRAMES, numPMEDSFrame=NUM_VIN_FRAMES,
                            maxPMEDSFactor=MAX_PME_DOWNSCALE});


if (FORMAT == "1080i") then
  -- Setup_VPP(vpp_instance, in_width, in_height, is_interlaced, is_420, pme_hscale, pme_vscale)
  Setup_VPP(lvpp0, HSIZE, VSIZE, 1, 0, 4, 4)
  lvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=563});
  lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=20});
  lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=HSTART_PIXEL});
  lvpp0:setParam({param=Q_LVPP_CMP_DEST_PROG, value=0});
elseif (FORMAT == "720p") then
  -- Setup_VPP(vpp_instance, in_width, in_height, is_interlaced, is_420, pme_hscale, pme_vscale)
  Setup_VPP(lvpp0, HSIZE, VSIZE, 0, 0, 4, 4)
  lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=25});
  lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=261});
elseif (FORMAT == "480i") then
  -- Setup_VPP(vpp_instance, in_width, in_height, is_interlaced, is_420, pme_hscale, pme_vscale)
  Setup_VPP(lvpp0, HSIZE, VSIZE, 0, 0, 2, 4)
  lvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=263});
  lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=22});
  lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=0});
  lvpp0:setParam({param=Q_LVPP_CMP_DEST_PROG, value=0});
end

lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=INPUT_PIXEL_AR_X});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=INPUT_PIXEL_AR_Y});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=NATIVE_FRAME_TIME});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=FRAME_TIME});

-- disable temporal filter for recon preview
lvpp0:setParam({param=Q_LVPP_CMP_HW_BYPASS_TFILTY, value=TF_LUMA_BYPASS});
lvpp0:setParam({param=Q_LVPP_CMP_HW_BYPASS_TFILTC, value=TF_CHROMA_BYPASS});

if (VIDEO_ONLY == 0) then
  lvpp0:setParam({param=Q_LVPP_CMP_AV_SYNCH_ENABLE, value=1});
end

lvpp0:activateCfg();


-- create and configure audio input
if (VIDEO_ONLY == 0) then
   -- setup audio input
   sain0 = sysctl0:sainCreate({name="sain0",port=0});
   if (AUDIO_STEREO == 1) then
      sain0:setParam({param=Q_SAIN_CMP_AUD_CHANNEL_NUM, value=Q_SAIN_CFP_CHANNEL_STEREO});
   else
      sain0:setParam({param=Q_SAIN_CMP_AUD_CHANNEL_NUM, value=Q_SAIN_CFP_CHANNEL_MONO_L});
   end
   sain0:setParam({param=Q_SAIN_CMP_AUD_SERIAL_MODE, value=Q_SAIN_CFP_AUD_SERIAL_MODE_I2S});
   sain0:setParam({param=Q_SAIN_CMP_AUD_MASTER_CLOCK, value=Q_SAIN_CFP_AUD_MASTER_CLOCK_256FS});
   sain0:setParam({param=Q_SAIN_CMP_AUD_SAMPLE_RATE, value=SAMPLERATE});
   sain0:setParam({param=Q_SAIN_CMP_AUD_SAMPLE_SIZE, value=16});
   sain0:setParam({param=Q_SAIN_CMP_AUD_FRAME_SIZE, value=AUDIO_FRAME_SIZE});
   sain0:activateCfg();

   -- create and bind audio encoder
   audenc0 = sysctl0:audencCreate({name="audenc0", ws=ws, codecType=AUDIO_CODEC});
   audenc0:setStreamID({streamid=AUDIOSTREAMID});
   audenc0:bindAudioInput({src=sain0});

   -- configure audio encoder
   if (AUDIO_STEREO == 1) then
      audenc0:setParam({param=Q_AUDENC_CMP_NUM_CHANNELS, value=Q_AUDENC_CFP_CHANNEL_STEREO});
   else
      audenc0:setParam({param=Q_AUDENC_CMP_NUM_CHANNELS, value=Q_AUDENC_CFP_CHANNEL_MONO});
   end
   audenc0:setParam({param=Q_AUDENC_CMP_SAMPLE_RATE, value=SAMPLERATE});
   audenc0:setParam({param=Q_AUDENC_CMP_SAMPLE_SIZE, value=16});
   audenc0:setParam({param=Q_AUDENC_CMP_BIT_RATE, value=ABITRATE});
   --codec specific configuration
   audenc0:setParam({param=Q_AUDENC_CMP_Q711_LAW, value=Q711_LAW});
   audenc0:setParam({param=Q_AUDENC_CMP_QAC_AUDIO_OBJECT_TYPE, value=QAC_AOT});
   audenc0:activateCfg();
end


-- create avc encoder
if (MAXNALUEN == 1) then
  avcenc0 = sysctl0:avcencCreate({name="avcenc0", ws=ws, maxwidth=HSIZE, maxheight=VSIZE16,
          numreferenceframes=NUM_AVC_REFERENCES, outputbuffersize=ENC_OUTPUT_BUFFER,
          outputAUs = VIDEO_AUS});
else
  avcenc0 = sysctl0:avcencCreate({name="avcenc0", ws=ws, maxwidth=HSIZE, maxheight=VSIZE16,
          numreferenceframes=NUM_AVC_REFERENCES, outputbuffersize=ENC_OUTPUT_BUFFER});
end

-- setup avc
--avcenc_VSIZEi60(avcenc, profile, level, bitrate_qp, gop_structure, picture_coding)
if (FORMAT == "1080i") then
  avcenc_1080i60(avcenc0, "high", 40, VBITRATE, gopstruct, FRAME_STRUCTURE)
elseif (FORMAT == "720p") then
  avcenc_720p60(avcenc0, "high", 40, VBITRATE, gopstruct, FRAME_STRUCTURE)
elseif (FORMAT == "480i") then
  avcenc_480i(avcenc0, "high", 31, VBITRATE, gopstruct, FRAME_STRUCTURE)
end

-- various other settings
avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, value=FRAME_TIME});
avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_SEI_PICT_TIMING_ENABLE, value=1});
avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_CLOSED_GOP, value=1});

-- Need SPS/PPS for streaming, even if we loop back due to socket overflow at startup
if (STREAMING > 0) then
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_NEW_SPS_EACH_IDR, value=1});
    avcenc0:activateVideoEncCfg();
end
        

-- max NALU settings
if (MAXNALUEN == 1) then

    guardband = MAX_NALU_LENGTH * 3 / 10;
    guardbandI = MAX_NALU_LENGTH * 4 / 10;
    

    print("target_nalu_len="..MAX_NALU_LENGTH.." guard="..guardband.." guard_I="..guardbandI.."\n");
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_FIXED_LENGTH_NALU_ENABLE, value=1});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_FIXED_NALU_LENGTH, value=MAX_NALU_LENGTH});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_FIXED_LENGTH_NALU_GUARD_I, value=guardbandI});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_FIXED_LENGTH_NALU_GUARD_NI, value=guardband});
    
    -- the settings below an example for max_nalu_length = 1400 and 720p30 with 3Mbps, they need to be
    -- tuned for other resolution,max_nalu_length and bitrate.  
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_FIXED_LENGTH_NALU_INIT_I, value=4});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_FIXED_LENGTH_NALU_INIT_NI, value=5});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_FIXED_LENGTH_NALU_CKPT_I_1, value=350});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_FIXED_LENGTH_NALU_CKPT_I_2, value=550});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_FIXED_LENGTH_NALU_CKPT_NI_1, value=350});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_FIXED_LENGTH_NALU_CKPT_NI_2, value=450});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_FIXED_LENGTH_NALU_QDEPTH_I_1, value=3});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_FIXED_LENGTH_NALU_QDEPTH_I_2, value=2});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_FIXED_LENGTH_NALU_QDEPTH_NI_1, value=6});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_FIXED_LENGTH_NALU_QDEPTH_NI_2, value=3});

    -- the below are usually necessary to ensure low exceeds
    --[[
    avcenc0:setVideoEncRateControlParam({param=Q_AVCENC_RC_CMP_COMPLEXITY_ENABLE, value=1});
    avcenc0:setVideoEncRateControlParam({param=Q_AVCENC_RC_CMP_I_DELTA_VARIABLE, value=0});
    --]]
else
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_FIXED_LENGTH_NALU_ENABLE, value=0});
end

-- force bitstream on
avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT, value=1});

avcenc0:activateVideoEncCfg();

avcenc0:bindVideoInput({src=lvpp0});

avcenc0:setStreamID({streamid=VIDEOSTREAMID});

-- ack every N frames to the AVC encoder
avcenc0:setAckInterval({interval=3});

-- create decoder

if (MAXNALUEN == 1) then
  avdec0 = sysctl0:avdecCreate({name="avdec0", atype=AUDIO_CODEC,
                                             vtype=Q_AVDEC_CFP_VIDEO_CODEC_AVC,
                                             audioBufferSize=100000,
                                             videoBufferSize=8000000,
                                             maxWidth=HSIZE,
                                             maxHeight=VSIZE,
                                             videoAUs=VIDEO_AUS,
					     numoutputframes=NUM_AVDEC_REFERENCES});
else
  avdec0 = sysctl0:avdecCreate({name="avdec0", atype=AUDIO_CODEC,
                                             vtype=Q_AVDEC_CFP_VIDEO_CODEC_AVC,
                                             audioBufferSize=100000,
                                             videoBufferSize=AVDEC_BUFFER_SIZE,
                                             maxWidth=HSIZE,
                                             maxHeight=VSIZE,
					     numoutputframes=NUM_AVDEC_REFERENCES});
end

rs0 = sysctl0:createReadStreamer({name="rs0"})

avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_ID, value=AUDIOSTREAMID});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_STREAM_ID, value=VIDEOSTREAMID});
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_RATE, value=SAMPLERATE});
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_SIZE, value=16});
if (AUDIO_STEREO == 1) then
    avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, value=Q_AVDEC_CFP_AUDIO_CHANNEL_STEREO});
else
    avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, value=Q_AVDEC_CFP_AUDIO_CHANNEL_MONO});
end
--avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_OUTPUT, value=1});
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_UNDERFLOW_THRESHOLD, value=500000});
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_UNDERFLOW_MARGIN, value=5000});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_OVERFLOW_THRESHOLD, value=50000});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_OVERFLOW_MARGIN, value=20000});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_UNDERFLOW_THRESHOLD, value=1000000});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_UNDERFLOW_MARGIN, value=10000});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_INITIAL_BUFFER_LEVEL, value=0});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_OUTPUT_TIMING, value=Q_AVDEC_CFP_VIDEO_OUTPUT_TIMING_LOW_DELAY});

avdec0:activateCfg();


-- done creating decoder

-- create display and grab render memory if composing
if (COMPOSEDISPLAY == 1) then
  avoc0 = sysctl0:avocCreate({name="avoc0", maxWidth=COMPWIDTH, maxHeight=COMPHEIGHT});
else
  avoc0 = sysctl0:avocCreate({name="avoc0"});
end

if (FORMAT == "1080i") then
  avoc0:setOutput1080i60({sync=vout_sync});
elseif (FORMAT == "720p") then
  avoc0:setOutput720p60();
elseif (FORMAT == "480i") then
  avoc0:setOutput480i();
end

-- Set composition parameters
if (COMPOSEDISPLAY == 1) then
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_X, value=COMPXOFFSET});
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_Y, value=COMPYOFFSET});
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=COMPWIDTH});
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=COMPHEIGHT});
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_WIDTH, value=COMPWIDTH});
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_HEIGHT, value=COMPHEIGHT});
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_BFF, value=INPUTBFF});
  avoc0:setParam({param=Q_AVOC_CMP_VPU_BG_COLOR, value=0x10808000});
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_ENABLE, value=1});
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_RATE_DIVIDER, value=COMPRATEDIVIDER});
else
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=HSIZE});
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=VSIZE});

  if (FORMAT == "1080i") then
    avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=1920});
    avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=1080});
  end

end

avoc0:activateCfg();

-- create and bind avdecoder avsn
avsn0 = sysctl0:avsnCreate({name="avsn0"});
avsn0:bindVideoInput({src=avdec0});

-- create and add video channels
vch0 = avoc0:addVideoChannel();
avoc0:setVideoChannelSource({ch=vch0, src=avsn0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=HSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=VSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
if (COMPOSEDISPLAY == 1) then
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=HSIZE/2});
else
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=HSIZE});
end
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=VSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=0});

if (VIDEO_ONLY == 0) then
   avoc0:setParam({param=Q_AVOC_CMP_AUD_SERIAL_MODE, value=Q_AVOC_CFP_AUD_SERIAL_MODE_I2S});
   avoc0:setParam({param=Q_AVOC_CMP_AUD_MASTER_CLOCK, value=Q_AVOC_CFP_AUD_MASTER_CLOCK_256FS});
   avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_RATE, value=SAMPLERATE});
   avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_SIZE, value=16});
end

avoc0:activateVideoChannelCfg();

if (VIDEO_ONLY == 0) then
    avsn0:bindAudioInput({src=avdec0});
    if (AUDIO_STEREO == 1) then
      ach0 = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_STEREO});
    else
      ach0 = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_MONO_L});
    end
    avoc0:setAudioChannelSource({ch=ach0, src=avsn0});
    avoc0:enableAudioChannel({ch=ach0, enable=1});
    avoc0:activateAudioChannelCfg();
end

if (COMPOSEDISPLAY == 1) then
  -- create and bind lvpp avsn
  avsn1 = sysctl0:avsnCreate({name="avsn1"});
  avsn1:bindVideoInput({src=lvpp0});

  vch1 = avoc0:addVideoChannel();
  avoc0:setVideoChannelSource({ch=vch1, src=avsn1});
  avoc0:setVideoChannelParam({ch=vch1, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=HSIZE});
  avoc0:setVideoChannelParam({ch=vch1, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=VSIZE});
  avoc0:setVideoChannelParam({ch=vch1, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=vch1, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
  avoc0:setVideoChannelParam({ch=vch1, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=HSIZE/2});
  avoc0:setVideoChannelParam({ch=vch1, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=VSIZE});
  avoc0:setVideoChannelParam({ch=vch1, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=HSIZE/2});
  avoc0:setVideoChannelParam({ch=vch1, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=0});
  avoc0:activateVideoChannelCfg();

  if (VIDEO_ONLY == 0) then
    -- echo audio to output
    avsn1:bindAudioInput({src=sain0});
    if (AUDIO_STEREO == 1) then
      ach0 = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_STEREO});
    else
      ach0 = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_MONO_L});
    end
    avoc0:setAudioChannelSource({ch=ach0, src=avsn1});
    avoc0:enableAudioChannel({ch=ach0, enable=1});
    avoc0:activateAudioChannelCfg();
  end

end

if(STREAMING == 0) then
  -- make the fifo
  os.execute("rm -f " .. FILENAME);
  os.execute("mkfifo -m 664 " .. FILENAME);
  -- open the bitstream writer
  ws:open({type="file", file=FILENAME, sid = {VIDEOSTREAMID}});
elseif(STREAMING == 1) then
  ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=0,max=%d", HOST, PORT, MAX_PACKET_SIZE), sid = {VIDEOSTREAMID}});
elseif(STREAMING == 2) then
 if (VIDEO_ONLY == 1) then
   ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=h264,aud=null,mod=%d,msz=1472", SERVER_IP, SERVER_PORT, SESSION_ID, SERVER_MODE), sid = {VIDEOSTREAMID, AUDIOSTREAMID}});
 elseif (AUDIO_ENCODER == "AAC-LC") then
   ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=h264,aud=aac,mod=%d,msz=1472", SERVER_IP, SERVER_PORT, SESSION_ID, SERVER_MODE), sid = {VIDEOSTREAMID, AUDIOSTREAMID}});
 elseif (AUDIO_ENCODER == "G711-A-LAW") then
   ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=h264,aud=g711a,mod=%d,msz=1472", SERVER_IP, SERVER_PORT, SESSION_ID, SERVER_MODE), sid = {VIDEOSTREAMID, AUDIOSTREAMID}});
 elseif (AUDIO_ENCODER == "G711-MU-LAW") then
   ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=h264,aud=g711u,mod=%d,msz=1472", SERVER_IP, SERVER_PORT, SESSION_ID, SERVER_MODE), sid = {VIDEOSTREAMID, AUDIOSTREAMID}});
 elseif (AUDIO_ENCODER == "G711") then
   ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=h264,aud=g711u,mod=%d,msz=1472", SERVER_IP, SERVER_PORT, SESSION_ID, SERVER_MODE), sid = {VIDEOSTREAMID, AUDIOSTREAMID}});
 end
end
-- start output
avoc0:start();
avsn0:start();

if (COMPOSEDISPLAY == 1) then
avsn1:start();
end

rs0:bindDecoder({dec=avdec0});
if(STREAMING > 0) then
  os.execute("sysctl -w net.core.rmem_default=1048576");  -- (default was 104448)
  os.execute("sysctl -w net.core.rmem_max=1048576");      -- (default was 104448)
end
if(STREAMING == 0) then
  bsr0 = rs0:open({type="qbox", file=FILENAME, srcId={AUDIOSTREAMID, VIDEOSTREAMID}});
elseif(STREAMING == 1) then
  bsr0 = rs0:open({type="udp", file=string.format("port=%s,bufferlen=%s", 9700, 1000000), srcId={VIDEOSTREAMID}});
elseif(STREAMING == 2) then
  os.execute("sleep 2");   -- allow time for RTP server to start
  bsr0 = rs0:open({type="rtp", file=string.format("sip=%s,prt=%d,dat=udp,ses=test,tlt=%d,tlo=%d,tlu=%d",SERVER_IP, SERVER_PORT,
  	TARGET_LATENCY_TOLERANCE, TARGET_LATENCY_OVERFLOW, TARGET_LATENCY_UNDERFLOW), srcId={VIDEOSTREAMID, AUDIOSTREAMID}});
end

-- start decoder
avdec0:start();

-- start file reading
rs0:play({bsr=bsr0});

-- start encoders
if (VIDEO_ONLY == 0) then
  audenc0:record();
end
avcenc0:record();

-- start audio/video input
lvpp0:start();
if (VIDEO_ONLY == 0) then
  sain0:start();
end


function ed()
  avcq = avcenc0:query({silent=1});
  avdq = avdec0:query({silent=1});
  framedelta = avcq["framesEncoded"] - avdq["videoFramesDecoded"];
  print("Encoder/Decoder frame count delta " .. framedelta .. "\n");
end

