--[[

This script is used to take JPEG snapshots from AVOC.

Objects are binded as shown in the following chart:

LVPP->(AVCENC)->AVSN->AVOC->NVPP->JPEGENC

where the AVCENC object is binded optionally.

--]]

----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

-- Number of AVSN channels
OUTCH=4;

-- Width and height of the output grid
OUTGRIDW=2;
OUTGRIDH=2;

-- AVC encoding bitrate
BITRATE_AVC         = BITRATE_AVC or {};
BITRATE_AVC[1]      = BITRATE_AVC[1] or 2000000;
BITRATE_AVC[2]      = BITRATE_AVC[2] or 2000000;
BITRATE_AVC[3]      = BITRATE_AVC[3] or 2000000;
BITRATE_AVC[4]      = BITRATE_AVC[4] or 2000000;

-- Picture quality of JPEG snapshots, 0 (worst) to 100 (best)
SNAPSHOT_QUALITY = SNAPSHOT_QUALITY or 50;

SNAPSHOT_FILENAME_PREFIX = SNAPSHOT_FILENAME_PREFIX or "snapshot"

-- Video output can be 1080i or 480i (note composite also is driven in svideo)
FORMAT          = FORMAT or '480i';

-- Video input can be svideo or composite
VIN             = VIN or "svideo";

-- Video output is set to HDMI for 1080i mode and svideo for 480i mode
VOUT            = VOUT or "svideo";

-- Configure streaming options for JPEG snapshots
--
-- When streaming = 0, snapshots will be saved on HDD with filenames that
-- has prefix defined by SNAPSHOT_FILENAME_PREFIX followed by an index number that
-- has 5 digits and starting from 00000.
--
-- When streaming = 1, snapshots will be streamed live to UDP
--
-- Host IP and base port for udp live stream
STREAMING       = STREAMING or 1;
HOST            = HOST or "192.168.40.215";
PORT            = PORT or 9700;
MAX_PACKET_SIZE = MAX_PACKET_SIZE or 1472;

-- Width and height of each live encoded stream
ENCW            = ENCW or 720;
ENCH            = ENCH or 480;

-- Number of reference frames, should be larger than 4 for recon preview,
-- exactly 4 for IP encode
NUM_AVC_REFERENCES = NUM_AVC_REFERENCES or 12;

-- Renderer field order.  This should match the field order of the input.
-- For example, DVD input is typically BFF.
RENDER_BFF = 0;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

-- Index of AVC live encoded channels
-- If enabled(1), that channel's AVSN object will use AVC's encoded frames instead of
-- direct feed from LVPP.
AVCENCCH    = AVCENCCH or {};
AVCENCCH[1] = 1;
AVCENCCH[2] = 1;
AVCENCCH[3] = 1;
AVCENCCH[4] = 1;

-- Enable the output port of the AVOC. When enabled the composited frames
-- will be sent. This allows the composited frames to be processed and
-- encoded by the downstream object(s).
AVOC_OUTPUT_PORT_ENABLE = 1;


BITSTREAM_OUTPUT=1;

if (FORMAT == "1080i") then
  DISPLAYWIDTH = 1920;
  DISPLAYHEIGHT = 1080;
  OUTW = 720;
  OUTH = 480;
elseif (FORMAT == "480i") then
  DISPLAYWIDTH = 720;
  DISPLAYHEIGHT = 480;
  OUTW=360;
  OUTH=240;
end


sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=RENDER_BFF, videocontrol=0x00101000, interleave=1});
sysctl0:setVPortMode({port=1, live=0});

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end

if (FORMAT == "1080i") then
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT, sync=vout_sync});
else
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
end

board.setVideoInput({port=0, format='480i', mode=VIN});

sysctl0:setVerboseLevel({param=1});

COMPWIDTH=OUTGRIDW*OUTW;
COMPHEIGHT=OUTGRIDH*OUTH;
COMPYOFFSET= (DISPLAYHEIGHT-(OUTGRIDH*OUTH))/2;
COMPXOFFSET= (DISPLAYWIDTH-(OUTGRIDW*OUTW))/2;

-- show or hide bitrate 0 for off, 1 for on
SHOWRATE = 0;

avoc0 = sysctl0:avocCreate({name="avoc0", maxWidth=COMPWIDTH, maxHeight=COMPHEIGHT});
if (FORMAT == "1080i") then
  avoc0:setOutput1080i60({sync=vout_sync});
elseif (FORMAT == "480i") then
  avoc0:setOutput480i();
end

avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_X, value=COMPXOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_Y, value=COMPYOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_ENABLE, value=1});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_BFF, value=RENDER_BFF});
avoc0:setParam({param=Q_AVOC_CMP_VPU_BG_COLOR, value=0x10808000});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_OUTPUT_PORT_ENABLE, value=AVOC_OUTPUT_PORT_ENABLE});
avoc0:activateCfg();

lvpp0 = sysctl0:lvppCreate({name="lvpp0", slot=0, port=0, maxWidth=ENCW, maxHeight=ENCH, numVidFrame=11, maxPMEDSFactor=1});

ws = sysctl0:createWriteStreamer({name="ws0"});
ws:showrate({enable=SHOWRATE});
ws:start()

Setup_VPP(lvpp0, 720, 480, 1, 0, 2, 4);
lvpp0:crop({cropWidth=720,cropHeight=480, horzOffset=0, vertOffset=0});
lvpp0:size({outputWidth=ENCW, outputHeight=ENCH});
lvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=263});
lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=0});
lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=22});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=3003});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=3003});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=8});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=9});
lvpp0:activateCfg();

-- CREATE ENCODERS
nvpp0 = {};
avcenc = {};
avsn = {};
videoStreamId = {};

for i = 1,4 do
   if (AVCENCCH[i] == 1) then
      avcenc[i] = sysctl0:avcencCreate({name="avcenc", ws=ws, maxwidth=ENCW, maxheight=ENCH, numreferenceframes=NUM_AVC_REFERENCES});
      videoStreamId = i + 2;
      
      avcenc_480i(avcenc[i], "high", 31, BITRATE_AVC[i], Q_AVCENC_CFP_GOP_STRUCTURE_IP, "frame")
      avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, value=3003});
      avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT, value=BITSTREAM_OUTPUT});
      avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_CLOSED_GOP, value=1});
      avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_NEW_SPS_EACH_IDR, value=1});
      avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_ENCODER_PREVIEW, value=1});
      avcenc[i]:activateVideoEncCfg();

      avcenc[i]:bindVideoInput({src=lvpp0});

      avcenc[i]:setStreamID({streamid=videoStreamId});
      avcenc[i]:setAckInterval({interval=8});
   end
end

for i = 1,OUTCH do
  avsn[i] = sysctl0:avsnCreate({name="avsn"});
end


nvpp0 = sysctl0:nvppCreate({name="nvpp0", maxWidth=ENCW, maxHeight=ENCH, maxPMEDSFactor=1});
-- workaround - crop instead of scale 2nd channel
Setup_VPP(nvpp0, ENCW, ENCH, 0, 0, 2, 2)
nvpp0:setParam({param=Q_NVPP_CMP_PICT_WIDTH,           value=ENCW});
nvpp0:setParam({param=Q_NVPP_CMP_PICT_HEIGHT,          value=ENCH});
nvpp0:setParam({param=Q_NVPP_CMP_CROP_OFFSET_X,        value=0});
nvpp0:setParam({param=Q_NVPP_CMP_CROP_OFFSET_Y,        value=0});
nvpp0:setParam({param=Q_NVPP_CMP_PIXEL_AR_X,           value=1});
nvpp0:setParam({param=Q_NVPP_CMP_PIXEL_AR_Y,           value=1});
nvpp0:setParam({param=Q_NVPP_CMP_PMEDS_H,              value=2});
nvpp0:setParam({param=Q_NVPP_CMP_PMEDS_V,              value=2});
nvpp0:setParam({param=Q_NVPP_CMP_STC_PER_FRAME,        value=3003});
nvpp0:setParam({param=Q_NVPP_CMP_STC_PER_OUTPUT_FRAME, value=3003});
nvpp0:setParam({param=Q_LVPP_CMP_PMEDS_EN,             value=0});
nvpp0:setParam({param=Q_NVPP_CMP_HW_BYPASS_TFILTC,     value=1});
nvpp0:activateCfg();
nvpp0:bindInput({src=avoc0});

vch = {};
x = 0;
y = 0;
for i = 1,OUTCH do
  vch[i] = avoc0:addVideoChannel();

  if AVCENCCH[i] == 0 then
    avsn[i]:bindVideoInput({src=lvpp0});
  else
    avsn[i]:bindVideoInput({src=avcenc[i]});
    avsn[i]:setParam({param=Q_AVSN_CMP_EXTERNAL_STC_OFFSET, value=-12012});
    avsn[i]:activateCfg();
  end
  avoc0:setVideoChannelSource({ch=vch[i], src=avsn[i]});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=ENCW});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=ENCH});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=OUTW});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=OUTH});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=x});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=y});

  x = x + OUTW;
  if x == COMPWIDTH then
    x = 0;
    y = y + OUTH;
  end
end
avoc0:activateVideoChannelCfg();

-- SNAPSHOT ENCODER
snapshotStreamId = 36;
snapshotenc = sysctl0:jpegencCreate({name="snapshotenc", ws=ws, maxwidth=ENCW, maxheight=ENCH, outputbuffersize=2000000});
snapshotenc:bindVideoInput({src=nvpp0});
snapshotenc:setParam({param=Q_JPEGENC_CMP_CAPTURE_INTERVAL, value=-1}); 
snapshotenc:setParam({param=Q_JPEGENC_CMP_CAPTURE_START_INDEX, value=0}); 
-- set to 0 to make it sit idle and wait for snapshot commands
snapshotenc:setParam({param=Q_JPEGENC_CMP_CAPTURE_MAX_ENCODED_FRAMES, value=0});
snapshotenc:setParam({param=Q_JPEGENC_CMP_BITSTREAM_BUFFER, value=1});
snapshotenc:activateCfg();

snapshotenc:setAckInterval({interval=1});
snapshotenc:setStreamID({streamid=snapshotStreamId});
snapshotenc:setQualityFactor({qualityfactor=SNAPSHOT_QUALITY});

-- How to present the snapshot outputs? --

-- jelem=1 to strip away qbox headers so that encoded frames can be viewed by ordinary jpeg viewers and VLC
if STREAMING == 1 then
   ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,jelem=1,max=%d", HOST, PORT, MAX_PACKET_SIZE), sid = {snapshotStreamId}});
elseif STREAMING == 0 then
   outFileStr = string.format("basefilename=%s,single=1,jelem=1", SNAPSHOT_FILENAME_PREFIX);
   ws:open({type="jpeg", file=outFileStr, sid={snapshotStreamId}});
end
	
-- START
snapshotenc:record();
nvpp0:start();
avoc0:start();

for i = 1,OUTCH do
   avsn[i]:start();
end

for i = 1,4 do
   if (AVCENCCH[i] == 1) then
      avcenc[i]:record();
   end
end

lvpp0:start();

function done()
  lvpp0:flush();

  for i=1,4 do
     if (AVCENCCH[i] == 1) then
	waitForEvents({objid=avcenc[i], timeout=1000000, any=0, events={Q_AVCENC_EV_BITSTREAM_FLUSHED}});
	break;
     end
  end

  os.execute("echo flushed");

end

-- query dump
function q()
  lvpp0:query();
  for i = 1,OUTCH do
     avsn[i]:query();
  end
  avoc0:query();
  nvpp0:query();
  snapshotenc:query();
  for i = 1,4 do
     if (AVCENCCH[i] == 1) then
	avcenc[i]:query();
     end
  end
end

-----------------------------
-- JPEG Snapshot Functions --
-----------------------------

-- Takes one snapshot
function shot()
   snapshotenc:snapshot({shots=1});
   print("Snapshot taken.\n");
end

-- Takes multiple snapshots
function shots(numShots)
   snapshotenc:snapshot({shots=numShots});
   print(numShots.." snapshots taken.\n");
end

function shotquality(q)
   snapshotenc:setQualityFactor({qualityfactor=q});
   print("JPEG snapshot quality changed to "..q..".\n");
end
