--[[

This script is used to decode up to 4 D1 videos with AVC or JPEG codec
selectable by the user.

Some restrictions for now:
1. Decode video only. No audio support.
2. Decode from files only. No streaming from UDP or RTP.
3. Files must be in QBOX format. 
4. Display in 1080i resolution only

--]]

----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------
-- Video output mode
-- "hdmi" HDMI output 1080i and 720p only)
-- "component" Component output
VOUT = VOUT or "component";

-- Video format
-- "1080i"
-- "720p"
-- "480i"
FORMAT = FORMAT or "1080i";

-- Number of decoding channels
DECCH = DECCH or 4;

-- Dimension of source videos; shoulde be D1
SRCW = SRCW or 720;
SRCH = SRCH or 480;

-- Width and height of the output display grid
OUTGRIDW    = OUTGRIDW or 2;
OUTGRIDH    = OUTGRIDH or 2;

-- Selects deocder type
-- "avc" AVC decoder
-- "jpg" JPEG decoder
VIDEO_CODEC = {};
VIDEO_CODEC[1] = VIDEO_CODEC[1] or "avc";
VIDEO_CODEC[2] = VIDEO_CODEC[2] or "avc";
VIDEO_CODEC[3] = VIDEO_CODEC[3] or "jpg";
VIDEO_CODEC[4] = VIDEO_CODEC[4] or "jpg";

-- Video stream IDs
VIDEO_STREAM_ID = {};
VIDEO_STREAM_ID[1] = VIDEO_STREAM_ID[1] or 1;
VIDEO_STREAM_ID[2] = VIDEO_STREAM_ID[2] or 2;
VIDEO_STREAM_ID[3] = VIDEO_STREAM_ID[3] or 3;
VIDEO_STREAM_ID[4] = VIDEO_STREAM_ID[4] or 4;

-- Decodes input file that has single stream or multiple streams
-- 0: Decodes ONE input file that has multiple streams
-- 1: Decodes multiple files; each of which has just one stream
SINGLE_STREAM_FILE = SINGLE_STREAM_FILE or 0;

-- Input file name(s)
if (SINGLE_STREAM_FILE == 0) then
   INPUTFILENAME    = INPUTFILENAME or "/directmnt/qa/streams/merlin/pirate.4D1.2avc.2mjpeg.qbox";
else
   INPUTFILENAME    = INPUTFILENAME or {};
   INPUTFILENAME[1] = INPUTFILENAME[1] or "/directmnt/qa/streams/merlin/pirate.vid1.480i.avc.qbox";
   INPUTFILENAME[2] = INPUTFILENAME[2] or "/directmnt/qa/streams/merlin/pirate.vid2.480i.avc.qbox";
   INPUTFILENAME[3] = INPUTFILENAME[3] or "/directmnt/qa/streams/merlin/pirate.vid3.480i.mjpeg.qbox";
   INPUTFILENAME[4] = INPUTFILENAME[4] or "/directmnt/qa/streams/merlin/pirate.vid4.480i.mjpeg.qbox";
end

-- Supported by both AVC and JPEG decoder
MEMORY_XFER_MODE = MEMORY_XFER_MODE or 0;

-- Low delay decoder options
-- Supported only by AVC decoder
-- 0 = disable low delay mode.
-- 1 = Low Delay mode, works with IPPP only bit streams.
-- 2 = Instant Display mode, works with IPPP streams only.
LOW_DELAY_MODE = LOW_DELAY_MODE or 0;

-- Video buffer size. Use bigger size for higher quality video.
VIDEO_BUFFER_SIZE = {};
VIDEO_BUFFER_SIZE[1] = VIDEO_BUFFER_SIZE[1] or 2097152;
VIDEO_BUFFER_SIZE[2] = VIDEO_BUFFER_SIZE[2] or 2097152;
VIDEO_BUFFER_SIZE[3] = VIDEO_BUFFER_SIZE[3] or 2097152;
VIDEO_BUFFER_SIZE[4] = VIDEO_BUFFER_SIZE[4] or 2097152;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

-- don't support audio for now
AUDIO_CODEC = Q_AVDEC_CFP_AUDIO_CODEC_NONE;

if (LOW_DELAY_MODE == 2) then
  if (MEMORY_XFER_MODE ~= 1) then
    print("Warning, Instant Display Mode is on, decoder is now forced to use DMA mode");
    MEMORY_XFER_MODE = 1;
  end
end

-- Initialize codec types depending on memory transfer mode
Q_VIDEO_CODEC = {};
if (MEMORY_XFER_MODE == 1) then
   for i = 1, DECCH do
      if (VIDEO_CODEC[i] == "avc") then
	 Q_VIDEO_CODEC[i] = Q_AVDEC_CFP_VIDEO_CODEC_AVC_INSTANT_DISPLAY;
      elseif (VIDEO_CODEC[i] == "jpg") then
	 Q_VIDEO_CODEC[i] = Q_AVDEC_CFP_VIDEO_CODEC_JPEG_INSTANT_DISPLAY;
      else
	 printf("Warning. Unknown codec type for channel %d.\n", i);
      end
   end
else
   for i = 1, DECCH do
      if (VIDEO_CODEC[i] == "avc") then
	 Q_VIDEO_CODEC[i] = Q_AVDEC_CFP_VIDEO_CODEC_AVC;
      elseif (VIDEO_CODEC[i] == "jpg") then
	 Q_VIDEO_CODEC[i] = Q_AVDEC_CFP_VIDEO_CODEC_JPEG;
      else
	 printf("Warning. Unknown codec type for channel %d.\n", i);
      end
   end   
end

-- Calculates the dimension of the composited display
if (FORMAT == "1080i") then
   COMPWIDTH   = OUTGRIDW * SRCW;
   COMPHEIGHT  = OUTGRIDH * SRCH;
   COMPYOFFSET = (1080 - (OUTGRIDH * SRCH)) / 2;
   COMPXOFFSET = (1920 - (OUTGRIDW * SRCW)) / 2;
else
   print("Only support 1080i display!\n");
end

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end

-- Set up the output port
if (FORMAT == "1080i") then
   board.setVideoOutput({port=0, format=FORMAT, mode=VOUT, sync=vout_sync});
else
   board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
end

-- Set up the size of the decoder memory partition. 
-- This partition is used by all audio and video decoders to store compressed bitstreams.
-- Therefore its size must be set to the sum of all decoders.
-- Align the buffer size to be 2048 bytes aligned.
totalBufferSize = 0;
for i = 1,DECCH do
   VIDEO_BUFFER_SIZE[i] = math.floor((VIDEO_BUFFER_SIZE[i] + 2047) / 2048);
   VIDEO_BUFFER_SIZE[i] = VIDEO_BUFFER_SIZE[i] * 2048;
   --printf("VIDEO_BUFFER_SIZE[%d] %d\n", i, VIDEO_BUFFER_SIZE[i]);
   totalBufferSize = totalBufferSize + VIDEO_BUFFER_SIZE[i];
end
sysctl0:configure({param=Q_SYS_CFG_DECODER_INPUT_PARTITION_SIZE, value=totalBufferSize});

-- Read Streamer
rs0 = sysctl0:createReadStreamer({name="rs0"});
rs0:setParam({param=Q_RS_CFG_SHOW_UPDATE, value=0});

-- AVOC

-- When doing composition, need to setup the non-live video input port
sysctl0:setVPortMode({port=1, live=0});

avoc0 = sysctl0:avocCreate({name="avoc0", maxWidth=COMPWIDTH, maxHeight=COMPHEIGHT});
avoc0:setOutput1080i60({sync=vout_sync});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_X, value=COMPXOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_Y, value=COMPYOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_ENABLE, value=1});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VPU_BG_COLOR, value=0x10808000}); --black
avoc0:activateCfg();

avoc0:fillRenderBuffers({yuv=0x00108080}); --black

print("AVOC created\n");

-- Create decoders
avdec = {};
avsndec = {};

for i = 1, DECCH do
   avdec[i] = sysctl0:avdecCreate({
				     name="avdec" .. VIDEO_STREAM_ID[i], 
				     atype=AUDIO_CODEC,
				     vtype=Q_VIDEO_CODEC[i], 
				     audioBufferSize=100000, 
				     videoBufferSize=VIDEO_BUFFER_SIZE[i], 
				     maxWidth=SRCW, 
				     maxHeight=SRCH,
				     ioMode=MEMORY_XFER_MODE
				  });

   avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_STREAM_ID, value=VIDEO_STREAM_ID[i]});
   avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_ID, value=0});
   avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_OVERRIDE, value=Q_AVDEC_CFP_TRUE});
   avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_DEFAULT, value=3003});

   if (VIDEO_CODEC[i] == "jpg") then
      avdec[i]:setParam({param=Q_AVDEC_CMP_JPEG_ASPECT_RATIO_WIDTH, value=8});
      avdec[i]:setParam({param=Q_AVDEC_CMP_JPEG_ASPECT_RATIO_HEIGHT, value=9});
   end
   avdec[i]:activateCfg();
    
   rs0:bindDecoder({dec=avdec[i]});
end

-- AVSN video channels
vch = {};
x = 0;
y = 0;

for i = 1, DECCH do
   avsndec[i] = sysctl0:avsnCreate({name="avsndec" .. i});
   vch[i] = avoc0:addVideoChannel();
   
   if (i <= DECCH) then
      avsndec[i]:bindVideoInput({src=avdec[i]});
      avoc0:setVideoChannelSource({ch=vch[i], src=avsndec[i]});
   end

   avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=SRCW});
   avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=SRCH});
   avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
   avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
   avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=SRCW});
   avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=SRCH});
   avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=x});
   avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=y});
   
   x = x + SRCW;
   if x == COMPWIDTH then
      x = 0;
      y = y + SRCH;
   end
end

avoc0:activateVideoChannelCfg();

-- Open read streamers
if (SINGLE_STREAM_FILE == 0) then
   bsr0 = rs0:open({type="qbox", file=INPUTFILENAME, srcId={VIDEO_STREAM_ID}});
   rs0:setReaderParam({bsr=bsr0, param=Q_RS_CFG_BSR_DMA_XFER_MODE, value=MEMORY_XFER_MODE});
else
   bsr = {};
   for i = 1, DECCH do
      bsr[i] = rs0:open({type="qbox", file=INPUTFILENAME[i], srcId={VIDEO_STREAM_ID[i]}});
      rs0:setReaderParam({bsr=bsr[i], param=Q_RS_CFG_BSR_DMA_XFER_MODE, value=MEMORY_XFER_MODE});
   end
end

-- Start decoding
avoc0:start();

for i = 1,DECCH do
    avsndec[i]:start();
end


for i = 1,DECCH do
   avdec[i]:start();
end

if (SINGLE_STREAM_FILE == 0) then
   rs0:play({bsr=bsr0});
else
   for i = 1,DECCH do
      rs0:play({bsr=bsr[i]});
   end
end


function q()
   avoc0:query();
   for i = 1,DECCH do
      avsndec[i]:query();
   end
   for i = 1,DECCH do
      avdec[i]:query();
   end
end
