#!/bin/sh

## simple script to run object tracking with DirectFB support.
## Parameters that can be passed to dfbobjtrack.so:
##      show_events     -- show all events with dimensions
##      tag_rect        -- tag each rectangle with a number
##      show_frame_event-- show the one and only frame event 
##      rect_thick      -- thickness of rectangles around moving
##                         objects, (0=solid rectangles, [1..8]=
##                         walls are 1..8 pixels thick)
##      reg_delay       -- wait number of seconds before registering
##                         callback.
##
##  All parameters that take arguments are of the form, <parameter>=<arg>
##
##  Usage: see usage() below
##

#-------------------------------------------------------------------------------
# The name of this script as invoked.
#-------------------------------------------------------------------------------
ME=`basename $0`

#-------------------------------------------------------------------------------
# Display an usage message to standard error.
#-------------------------------------------------------------------------------

usage()
{
    1>&2 cat << END_USAGE_MSG
Usage: ${ME} [options]

Shell script to run object tracking lua scripts with DirectFB support.
Since many things need to be set up in addition to running the lua
script, it is much simplier to just use this script instead of calling
the lua scripts directly.  Options are:

-i X    Specify input (from file or concurrent sessions from live
        source).  Numbers {1, 2, 3, 4} indicate the number of
        concurrent object tracking instances to run from a single live
        source. Anything else specifies an input file name. If -i is
        not specified, the default is a single channel of object
        tracking from a live source.  For -i {2, 3, 4}, the input must
        be 480i.  

-f X    Specify input/output format.  Options are one of {480i, 576i,
        720p, 720p30, 1080i, 1080p30}.  Default is 480i.

-a X    Specify the analytic streaming option (1 or 2) as documented 
        in Lua script. Default is 0.

-mgappdir X     Specify the mgapp directory if not using from
                /usr/bin. Should be the directory above the bin 
                directory.

Examples:

    Run from a live 480i input source and output to a 480i display:
    ${ME} 

    Run from a file and output to a 480i display:
    ${ME} -i video_file.yuv

    Run multiple tracking sessions from live 480i source and display:
    ${ME} -i 4
    
    Run from a live HD source and display HD:
    ${ME} -f 720p


END_USAGE_MSG
}

#-------------------------------------------------------------------------------
# Parse all command line argvs until the first one that does not begin
# with the '-' character.
#-------------------------------------------------------------------------------

# defaults
INPUT=1
FORMAT=480i
ENABLE_ANALYTICS_STREAMING=0
MGAPPDIR=

while [ $# -ge 1 ] ; do
    case "${1}" in
    -h | --h | -help | --help)
        usage
        shift
        exit 0
        ;;
    -i)
        shift
        INPUT=${1}
        shift
        ;;
    -f)
        shift
        FORMAT=${1}
        shift
        ;;
    -a)
        shift
        ENABLE_ANALYTICS_STREAMING=${1}
        shift
        ;;
    -mgappdir)
        shift
        MGAPPDIR=${1}
        shift
        ;;
    -*)
        1>&2 echo "${ME}: Unsupported option: ${1}"
        usage
        shift
        exit 1
        ;;
    *)
        break
        ;;
    esac
done

if [ "X${MGAPPDIR}" != "X" ]
then
    export LD_LIBRARY_PATH=$MGAPPDIR/lib:$MGAPPDIR/lib/plugins:/usr/lib:/usr/lib/plugins:${LD_LIBRARY_PATH}
fi

#-------------------------------------------------------------------------------
# setup call to lua script
#-------------------------------------------------------------------------------

# set script to use and initial parameters
SCRIPTDIR=`dirname $0`
if [ "${INPUT}" = "1" ]; then
    SCRIPT="${SCRIPTDIR}/obj_track.lua -e FORMAT='${FORMAT}' ; ENABLE_ANALYTICS_STREAMING =${ENABLE_ANALYTICS_STREAMING} ; OBJECTTRACKING_DISP=1 "
elif [ "${INPUT}" = "2" -o "${INPUT}" = "3" -o "${INPUT}" = "4" ]; then
    SCRIPT="${SCRIPTDIR}/encode_4D1_from_1NTSC.lua -e FORMAT='${FORMAT}' ; ENCCH='${INPUT}' ; ENABLE_ANALYTICS_STREAMING =${ENABLE_ANALYTICS_STREAMING} ; OBJECTTRACKING_DISP=1 "
else
    SCRIPT="${SCRIPTDIR}/obj_track_file.lua -e FORMAT='${FORMAT}' ; FILEIN='${INPUT}' "
fi

# add extra params as necessary
if [ "${FORMAT}" = '720p' -o  "${FORMAT}" = '1080i' ]; then
    SCRIPT="${SCRIPT} ; VIN='component' ; VOUT='hdmi' "
fi
if [ "${FORMAT}" = '720p30' -o  "${FORMAT}" = '1080p30' ]; then
    SCRIPT="${SCRIPT} ; VIN='component' ; VOUT='hdmi' ; HALF_FRAMERATE = 1 "
fi

# set final run string, run lua script and exit
T_CFG=/tmp/mgs
trap "rm ${T_CFG} ; exit 0" SIGINT SIGKILL
echo "mobiapp.so mobiapp -f ${SCRIPT}" >> ${T_CFG}
echo "ui_lua.so ui_lua -f ${SCRIPT}" >> ${T_CFG}
echo "dfbobjtrack.so" >> ${T_CFG}
mgapp -c ${T_CFG}
rm ${T_CFG}

exit 0
